#!/usr/bin/env python3
# coding=utf-8
# author: @netmanchris
# -*- coding: utf-8 -*-
"""
This module contains class objects for working with the specific types of network
infrastructure equipment in the HPE IMC NMS using the RESTful API. These objects
rely on various other functions from within this library.

"""

# This section imports required libraries
from pyhpeimc.plat.device import *
from pyhpeimc.plat.alarms import *
from pyhpeimc.plat.netassets import *
from pyhpeimc.plat.termaccess import *
from pyhpeimc.plat.vlanm import *

from pyhpeimc.auth import HEADERS


# IMC Device Class


class IMCDev:
    """
    imc_dev class takes in the ip_address which is used as the primary key to gather
    the following attributes
    for a device which as been previously discovered in the HP IMC Network Management
    platform.

     Each instance of this class should have the following attributes

     ip: The IP address used to manage the device in HP IMC
     description: returns the description of the device as discovered in HP IMC
     location: returns the location of the device as discovered in HP IMC
     contact: returns the contact of the device as discovered in HP IMC
     type: returns the type of the device as discovered in HP IMC
     name: returns the name of the device as discovered in HP IMC
     status: returns the current alarm status as discovered in HP IMC
     devid: returns the current devid used to internally identify the device
     as discovered in HP IMC
     interfacelist: returns the current list of interfaces for the device as
     discovered in HP IMC
     numinterface: returns a count of the number of interfaces in the interfacelist
     attribute
     vlans: returns the current vlans existing in the device as discovered in HP IMC.
     Device must be supported in the HP IMC Platform VLAN manager module.
     accessinterfaces: returns the device interfaces configured as access interfaces.
     Device must be supported in the HP IMC Platform VLAN manager module.
     trunkinterfaces: returns the device interfaces configured as trunk interfaces.
     Device must be supported in the HP IMC Platform VLAN manager module.
     alarm: returns the current unrecovered alarms as known by HP IMC.
     num alarms: returns a count of the number of alarms as returned by the alarm
     attribute
     serial: returns the network assets, including serial numbers for the device as
     discovered by HP IMC. The device must support the ENTITY MIB ( rfc 4133 ) for
     this value to be returned.
     runconfig: returns the most recent running configuration for the device as known
     by HP IMC. The device must be be supported in the HP IMC platform ICC module.
     startconfig: returns the most recent startup configuration for the device as known
     by HP IMC. The device must be supported in the HP IMC platform ICC module.
     ipmacarp: returns the current device maciparp table as discovered by HP IMC.

     The imc_dev class supports the following methods which can be called upon an instance of
     this class

     getvlans: This method executes the getvlans function on the specific instance of the imc_dev
     object and populates the return into the self.vlans attribute. Devices must be supported in
     the HPE IMC Platform VLAN Manager module
     addvlan: This method executes the addvlan function on the specific instance of the imc_dev
     object. Devices must supported in the HP IMC Platform VLAN Manager module.

     """


    def __init__(self, ip_address, auth, url):
        """
        Function take in input of ipv4 address, auth and url and returns and object of type IMCDev
        :param ip_address: valid IPv4 address
        :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class
        :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass
        :return IMCDev object
        :rtype IMCDev
        """
        self.auth = auth
        self.url = url
        self.dev_details = get_dev_details(ip_address, auth, url)
        self.ip = self.dev_details['ip']
        self.description = self.dev_details['sysDescription']
        self.location = self.dev_details['location']
        self.contact = self.dev_details['contact']
        self.type = self.dev_details['typeName']
        self.name = self.dev_details['sysName']
        self.status = self.dev_details['statusDesc']
        self.devid = self.dev_details['id']
        self.interfacelist = get_all_interface_details(auth, url, devip=self.ip)
        self.numinterface = len(self.interfacelist)
        self.vlans = get_dev_vlans(auth, url, devid=None, devip=self.ip)
        self.accessinterfaces = get_device_access_interfaces(auth, url, devip=self.ip)
        self.trunkinterfaces = get_trunk_interfaces(auth, url, devip = self.ip)
        self.alarm = get_dev_alarms(auth, url, devip = self.ip)
        self.numalarm = len(get_dev_alarms(auth, url, devip= self.ip))
        self.assets = get_dev_asset_details(self.ip, auth, url)
        self.serials = [({'name': asset['name'], 'serialNum': asset['serialNum']}) for asset in
                         self.assets]
        self.runconfig = get_dev_run_config(auth, url, devip = self.ip)
        self.startconfig = get_dev_start_config(auth, url, devip = self.ip)
        self.ipmacarp = get_ip_mac_arp_list(auth, url, devip = self.ip)

    def getvlans(self):
        """
        Function operates on the IMCDev object and updates the vlans attribute
        :return:
        """
        self.vlans = get_dev_vlans( self.auth, self.url, devid=self.devid)

    def addvlan(self, vlanid, vlan_name):
        """
        Function operates on the IMCDev object. Takes input of vlanid (1-4094), str of vlan_name,
        auth and url to execute the create_dev_vlan method on the IMCDev object. Device must be
        supported in the HPE IMC Platform VLAN Manager module.
        :param vlanid: str of VLANId ( valid 1-4094 )
        :param vlan_name: str of vlan_name
        :return:
        """
        create_dev_vlan( vlanid, vlan_name, self.auth, self.url, devid = self.devid)

    def delvlan(self, vlanid):
        """
        Function operates on the IMCDev object. Takes input of vlanid (1-4094),
        auth and url to execute the delete_dev_vlans method on the IMCDev object. Device must be
        supported in the HPE IMC Platform VLAN Manager module.
        :param vlanid: str of VLANId ( valid 1-4094 )
        :return:
        """
        delete_dev_vlans( vlanid, self.auth, self.url, devid = self.devid)

    def getipmacarp(self):
        """
        Function operates on the IMCDev object and updates the ipmacarp attribute
        :return:
        """
        self.ipmacarp = get_ip_mac_arp_list(self.auth, self.url, devid = self.devid)


class IMCInterface:
    """
    Class instantiates an object to gather and manipulate attributes and methods of a single
    interface on a single infrastructure device, such as a switch or router.
    """

    def __init__(self, ip_address, ifindex, auth, url):
        self.auth = auth
        self.url = url
        self.dev_details = get_dev_details(ip_address, auth, url)
        self.ip = self.dev_details['ip']
        self.devid = self.dev_details['id']
        self.description = self.dev_details['sysDescription']
        self.location = self.dev_details['location']
        self.contact = self.dev_details['contact']
        self.type = self.dev_details['typeName']
        self.name = self.dev_details['sysName']
        self.status = self.dev_details['statusDesc']
        self.interface_details = get_interface_details(ifindex, self.auth, self.url, devip=self.ip)
        self.ifIndex = self.interface_details['ifIndex']
        self.macaddress = self.interface_details[
            'phyAddress']
        self.status = self.interface_details['statusDesc']
        self.adminstatus = self.interface_details[
            'adminStatusDesc']
        self.name = self.interface_details['ifDescription']
        self.description = self.interface_details[
            'ifAlias']
        self.mtu = self.interface_details['mtu']
        self.speed = self.interface_details['ifspeed']
        self.accessinterfaces = get_device_access_interfaces(self.auth, self.url, devip = self.ip)
        self.pvid = get_access_interface_vlan(self.ifIndex, self.accessinterfaces)
        self.lastchange = self.interface_details['lastChange']

    def down(self):
        """
        Function operates on the IMCInterface object and configures the interface into an
        administratively down state and refreshes contents of self.adminstatus
        :return:
        """
        set_interface_down(self.ifIndex, self.auth, self.url, devip=self.ip)
        self.adminstatus = get_interface_details(self.ifIndex, self.auth, self.url, devip=self.ip)[
            'adminStatusDesc']

    def up(self):
        """
                Function operates on the IMCInterface object and configures the interface into an
                administratively up state and refreshes contents of self.adminstatus
                :return:
                """
        set_interface_up(self.ifIndex, self.auth, self.url, devip=self.ip)
        self.adminstatus = get_interface_details(self.ifIndex, self.auth, self.url, devip=self.ip)[
            'adminStatusDesc']


# TODO refactor deallocateIp method for human consumption
# TODO Add real_time_locate functionality to nextfreeip method to search IP address before offering

class IPScope:
    """
        Class instantiates an object to gather and manipulate attributes and methods of a IP
        scope as configured in the HPE IMC Platform Terminal Access module. Note: IPScope must already exist.
        """

    def __init__(self, netaddr, auth, url):
        self.id = get_scope_id(netaddr, auth, url)
        self.details = get_ip_scope_detail(auth, url, self.id)
        self.hosts = get_ip_scope_hosts(auth, url, self.id)
        self.auth = auth
        self.url = url
        self.netaddr = ipaddress.ip_network(netaddr)
        self.startip = get_ip_scope_detail(auth, url, self.id)['startIp']
        self.endip = get_ip_scope_detail(auth, url, self.id)['endIp']
        if 'assignedIpScope' in self.details:
            self.child = get_ip_scope_detail(auth, url, self.id)['assignedIpScope']

    def allocate_ip(self, hostipaddress, name, description):
        """
        Object method takes in input of hostipaddress, name and description and adds them to the
        parent ip scope.
        :param hostipaddress: str of ipv4 address of the target host ip record
        :param name: str of the name of the owner of the target host ip record
        :param description: str of a description of the target host ip record
        :return:
        """
        add_scope_ip(hostipaddress, name, description, self.auth, self.url, scopeid=self.id)

    def deallocate_ip(self, hostipaddress):
        """
        Object method takes in input of hostip address,removes them from the parent ip scope.
        :param hostid: str of the hostid of  the target host ip record

        :return:
        """
        delete_host_from_segment(hostipaddress, self.netaddr, self.auth, self.url)

    def gethosts(self):
        """
        Method gets all hosts currently allocated to the target scope and refreashes the self.hosts
        attributes of the object
        :return:
        """
        self.hosts = get_ip_scope_hosts(self.auth, self.url, self.id)

    def nextfreeip(self):
        """
        Method searches for the next free ip address in the scope object and returns it as a str
        value.
        :return:
        """
        allocated_ips = [ipaddress.ip_address(host['ip']) for host in self.hosts]
        for ip in self.netaddr:
            if str(ip).split('.')[-1] == '0':
                continue
            if ip not in allocated_ips:
                return ip

    def addchild(self, startip, endip, name, description):
        """
        Method takes inpur of str startip, str endip, name, and description and adds a child scope.
        The startip and endip MUST be in the IP address range of the parent scope.
        :param startip: str of ipv4 address of the first address in the child scope
        :param endip: str of ipv4 address of the last address in the child scope
        :param name: of the owner of the child scope
        :param description: description of the child scope
        :return:
        """
        add_child_ip_scope(self.auth, self.url, startip, endip, name, description, self.id)



#!/usr/bin/env python3
# coding=utf-8
# author: @netmanchris
# -*- coding: utf-8 -*-
"""
This module contains functions for working with the alarms capabilities
of the HPE IMC NMS platform using the RESTful API

"""

# This section imports required libraries
import json

import requests

from pyhpeimc.auth import HEADERS
from pyhpeimc.plat.device import get_dev_details


def get_dev_alarms(auth, url, devid=None, devip=None):
    """
    function takes the devId of a specific device and issues a RESTFUL call to get the current
    alarms for the target device.

    :param devid: int or str value of the target device

    :param devip: str of ipv4 address of the target device

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return:list of dictionaries containing the alarms for this device

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.alarms import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> dev_alarms = get_dev_alarms(auth.creds, auth.url, devip='10.101.0.221')

    >>> assert 'ackStatus' in dev_alarms[0]

    """
    # checks to see if the imc credentials are already available
    if devip is not None:
        devid = get_dev_details(devip, auth, url)['id']
    f_url = url + "/imcrs/fault/alarm?operatorName=admin&deviceId=" + \
                        str(devid) + "&desc=false"
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            dev_alarm = (json.loads(response.text))
            if 'alarm' in dev_alarm:
                return dev_alarm['alarm']
            else:
                return "Device has no alarms"
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' get_dev_alarms: An Error has occured'


def get_realtime_alarm(username, auth, url):
    """Takes in no param as input to fetch RealTime Alarms from HP IMC RESTFUL API

    :param username OpeatorName, String type. Required. Default Value "admin". Checks the operator
     has the privileges to view the Real-Time Alarms.

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return:list of dictionaries where each element of the list represents a single alarm as
    pulled  from the the current list of realtime alarms in the HPE IMC Platform

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.alarms import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> real_time_alarm = get_realtime_alarm('admin', auth.creds, auth.url)

    >>> assert type(real_time_alarm) is list

    >>> assert 'faultDesc' in real_time_alarm[0]

    """
    f_url = url + "/imcrs/fault/faultRealTime?operatorName=" + username
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        realtime_alarm_list = (json.loads(response.text))
        return realtime_alarm_list['faultRealTime']['faultRealTimeList']
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' get_realtime_alarm: An Error has occured'


def get_alarms(username, auth, url):
    """Takes in no param as input to fetch RealTime Alarms from HP IMC RESTFUL API

    :param username OpeatorName, String type. Required. Default Value "admin". Checks the operator
     has the privileges to view the Real-Time Alarms.

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return:list of dictionaries where each element of the list represents a single alarm as
    pulled  from the the current list of browse alarms in the HPE IMC Platform

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.alarms import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> all_alarms = get_alarms('admin', auth.creds, auth.url)

    >>> assert (type(all_alarms)) is list

    >>> assert 'ackStatus' in all_alarms[0]

    """
    f_url = url + "/imcrs/fault/alarm?operatorName=" + username + \
                     "&recStatus=0&ackStatus=0&timeRange=0&size=50&desc=true"
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            alarm_list = (json.loads(response.text))
            return alarm_list['alarm']
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' get_alarms: An Error has occured'

def get_alarm_details(alarm_id, auth, url):
    """
    function to take str input of alarm_id, issues a REST call to the IMC REST interface and
    returns a dictionary object which contains the  details of a specific alarm.
    :param alarm_id: str number which represents the internal ID of a specific alarm
    :param auth:
    :param url:
    :return:
    """
    f_url = url + "/imcrs/fault/alarm/" + str(alarm_id)
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        alarm_details = json.loads(response.text)
        return alarm_details
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' get_alarm_details: An Error has occured'


def acknowledge_alarm(alarm_id, auth, url):
    """
    Function tasks input of str of alarm ID and sends to REST API. Function will acknowledge
    designated alarm in the IMC alarm database.
    :param alarm_id: str of alarm ID
    param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass
    :return: integer HTTP response code

    :rtype int
    """
    f_url = url + "/imcrs/fault/alarm/acknowledge/"+str(alarm_id)
    response = requests.put(f_url, auth=auth, headers=HEADERS)
    try:
        return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' get_alarms: An Error has occured'


def recover_alarm(alarm_id, auth, url):
    """
    Function tasks input of str of alarm ID and sends to REST API. Function will acknowledge
    designated alarm in the IMC alarm database.
    :param alarm_id: str of alarm ID
    param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass
    :return: integer HTTP response code

    :rtype int
    """
    f_url = url + "/imcrs/fault/alarm/recover/" + str(alarm_id)
    response = requests.put(f_url, auth=auth, headers=HEADERS)
    try:
        return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + 'recover_alarm: An Error has occured'





#!/usr/bin/env python3
# coding=utf-8
# author: @netmanchris
# -*- coding: utf-8 -*-
"""
This module contains functions for working with the performance management
capabilities of the HPE IMC NMS platform using the RESTful API

"""

# This section imports required libraries
import json

import requests

from pyhpeimc.auth import HEADERS


def add_perf_task(task, auth, url):
    """
    function takes the a python dict containing all necessary fields for a performance tasks,
    transforms the dict into JSON and issues a RESTFUL call to create the performance task. device.

    :param task: dictionary containing all required fields for performance tasks

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: 204

    :rtype: str

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.perf import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> new_task = {'indexDesc': '1.3.6.1.4.1.9.9.13.1.3.1.3',
                    'indexType': '[index1[0]:ciscoEnvMonTemperatureStatusValue:1:0]',
                    'itemFunction': '1.3.6.1.4.1.9.9.13.1.3.1.3',
                    'itemName': 'Cisco_Temperature',
                    'selectDefaultUnit': '400',
                    'unit': 'Celsius'}

    >>> new_perf_task = add_perf_task(new_task, auth.creds, auth.url)
    """
    add_perf_task_url = "/imcrs/perf/task"
    f_url = url + add_perf_task_url
    payload = json.dumps(task)
    response = requests.post(f_url, data=payload, auth=auth, headers=HEADERS)
    try:
        return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' add_perf_task: An Error has occured'


def get_perf_task(task_name, auth, url):
    """
        function takes the a str object containing the name of an existing performance tasks and
        issues a RESTFUL call to the IMC REST service. It will return a list

        :param task_name: str containing the name of the performance task

        :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

        :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

        :return: 204

        :rtype: dict

        >>> from pyhpeimc.auth import *

        >>> from pyhpeimc.plat.perf import *

        >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

        >>> selected_task = get_perf_task('Cisco_Temperature', auth.creds, auth.url)

        >>> assert type(selected_task) is dict

        >>> assert 'taskName' in selected_task
        """
    get_perf_task_url = "/imcrs/perf/task?name=" + task_name + "&orderBy=taskId&desc=false"
    f_url = url + get_perf_task_url
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            perf_task_info = (json.loads(response.text))
            if 'task' in perf_task_info:
                perf_task_info = (json.loads(response.text))['task']
            else:
                perf_task_info = "Task Doesn't Exist"
            return perf_task_info
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' get_perf_task: An Error has occured'


def delete_perf_task(task_name, auth, url):
    """
    Function takes a str of the target task_name to be deleted and retrieves task_id using
    the get_perf_task function. Once the task_id has been successfully retrieved it is
    populated into the task_id variable and an DELETE call is made against the HPE IMC REST
    interface to delete the target task.
    :param task_name: str of task name
    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: int of 204 if successful, str of "Perf Task doesn't exist" i

    :rtype: int

    """
    task_id = get_perf_task(task_name, auth, url)
    if isinstance(task_id, str):
        print("Perf task doesn't exist")
        return 403
    task_id = task_id['taskId']
    get_perf_task_url = "/imcrs/perf/task/delete/" + str(task_id)
    f_url = url + get_perf_task_url
    response = requests.delete(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 204:
            print("Perf Task successfully delete")
            return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' delete_perf_task: An Error has occured'



#!/usr/bin/env python3
# coding=utf-8
# author: @netmanchris
# -*- coding: utf-8 -*-
"""
This module contains functions for working with the icc (configuration
management) capabilities of the HPE IMC NMS platform using the RESTful API

"""

# This section imports required libraries
import json

import requests

from pyhpeimc.auth import HEADERS


def get_cfg_template(auth, url, folder=None):
    """
    Function takes no input and returns a list of dictionaries containing the configuration
    templates in the root folder of the icc configuration template library.

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param folder: optional str of name of target folder

    :folder = str of target folder name

    :return: List of Dictionaries containing folders and configuration files in the ICC library.

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.icc import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> config_templates = get_cfg_template(auth.creds, auth.url)

    >>> assert type(config_templates) is list

    >>> assert 'confFileName' in config_templates[0]


    >>> config_templates_folder = get_cfg_template(auth.creds, auth.url, folder='ADP_Configs')

    >>> assert type(config_templates_folder) is list

    >>> assert 'confFileName' in config_templates_folder[0]

    >>> config_template_no_folder = get_cfg_template(auth.creds, auth.url, folder='Doesnt_Exist')

    >>> assert config_template_no_folder is None
    """
    if folder is None:
        get_cfg_template_url = "/imcrs/icc/confFile/list"
    else:
        folder_id = get_folder_id(folder, auth, url)
        get_cfg_template_url = "/imcrs/icc/confFile/list/" + str(folder_id)
    f_url = url + get_cfg_template_url
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            cfg_template_list = (json.loads(response.text))['confFile']
            if isinstance(cfg_template_list, list):
                return cfg_template_list
            elif isinstance(cfg_template_list, dict):
                return [cfg_template_list]
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_cfg_template: An Error has occured"


def create_cfg_segment(filename, filecontent, description, auth, url):
    """
    Takes a str into var filecontent which represents the entire content of a configuration
    segment, or partial configuration file. Takes a str into var description which represents the
    description of the configuration segment
    :param filename: str containing the name of the configuration segment.

    :param filecontent: str containing the entire contents of the configuration segment

    :param description: str contrianing the description of the configuration segment

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: If successful, Boolena of type True

    :rtype: Boolean

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.icc import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> filecontent = 'sample file content'

    >>> create_new_file = create_cfg_segment('CW7SNMP.cfg',
                                              filecontent,
                                              'My New Template',
                                               auth.creds,
                                               auth.url)

    >>> template_id = get_template_id('CW7SNMP.cfg', auth.creds, auth.url)

    >>> assert type(template_id) is str

    >>>
    """
    payload = {"confFileName": filename,
               "confFileType": "2",
               "cfgFileParent": "-1",
               "confFileDesc": description,
               "content": filecontent}
    f_url = url + "/imcrs/icc/confFile"
    response = requests.post(f_url, data=(json.dumps(payload)), auth=auth, headers=HEADERS)
    try:
        if response.status_code == 201:
            print("Template successfully created")
            return response.status_code
        elif response.status_code is not 201:
            return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " create_cfg_segment: An Error has occured"


def get_template_id(template_name, auth, url):
    """
    Helper function takes str input of folder name and returns str numerical id of the folder.
    :param template_name: str name of the target template

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: str numerical id of the folder

    :rtype: str

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.icc import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> file_id = get_template_id('CW7SNMP.cfg', auth.creds, auth.url)

    >>> assert type(file_id) is int

    """
    object_list = get_cfg_template(auth=auth, url=url)
    for template in object_list:
        if template['confFileName'] == template_name:
            return int(template['confFileId'])
    return "template not found"


def get_folder_id(folder_name, auth, url):
    """
    Helper function takes str input of folder name and returns str numerical id of the folder.
    :param folder_name: str name of the folder

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: str numerical id of the folder

    :rtype: str

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.icc import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> default_folder_id = get_folder_id('Default Folder', auth.creds, auth.url)

    >>> assert type(default_folder_id) is int

    """
    object_list = get_cfg_template(auth=auth, url=url)
    for template in object_list:
        if template['confFileName'] == folder_name:
            return int(template['confFileId'])
    return "Folder not found"


def delete_cfg_template(template_name, auth, url):
    """Uses the get_template_id() funct to gather the template_id
    to craft a url which is sent to the IMC server using a Delete Method
    :param template_name: str containing the entire contents of the configuration segment

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: If successful, return int of status.code 204.

    :rtype: Int

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.icc import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> delete_cfg_template('CW7SNMP.cfg', auth.creds, auth.url)


    """
    file_id = get_template_id(template_name, auth, url)
    f_url = url + "/imcrs/icc/confFile/" + str(file_id)
    response = requests.delete(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 204:
            print("Template successfully Deleted")
            return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " delete_cfg_template: An Error has occured"


def get_template_details(template_name, auth, url):
    """Uses the get_template_id() funct to gather the template_id to craft a
    get_template_details_url which is sent to the IMC server using
    a get Method
    :param template_name: str containing the entire contents of the configuration segment

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: If successful, return dict containing the template details

    :rtype: dict

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.icc import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> filecontent = 'sample file content'

    >>> create_new_file = create_cfg_segment('CW7SNMP.cfg',
                                              filecontent,
                                              'My New Template',
                                               auth.creds,
                                               auth.url)

    >>> template_contents = get_template_details('CW7SNMP.cfg', auth.creds, auth.url)

    >>> assert type(template_contents) is dict

    """
    file_id = get_template_id(template_name, auth, url)
    if isinstance(file_id, str):
        return file_id
    f_url = url + "/imcrs/icc/confFile/" + str(file_id)
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            template_details = json.loads(response.text)
            return template_details
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_template_contents: An Error has occured"



#!/usr/bin/env python3
# coding=utf-8
# author: @netmanchris
# -*- coding: utf-8 -*-
"""
This module contains functions for working with the VLAN Manager capabilities
of the HPE IMC NMS platform using the RESTful API

"""

# This section imports required libraries
import json

import requests

from pyhpeimc.auth import HEADERS
from pyhpeimc.plat.device import get_dev_details

#pylint: disable=R0913
# This section contains functions which operate at the device level


def get_dev_vlans(auth, url, devid=None, devip=None):
    """Function takes input of devID to issue RESTUL call to HP IMC

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param devid: str requires devId as the only input parameter

    :param devip: str of ipv4 address of the target device

    :return: list of dictionaries where each element of the list represents one vlan on the
    target device

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.vlanm import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> vlans = get_dev_vlans('350', auth.creds, auth.url)

    >>> assert type(vlans) is list

    >>> assert 'vlanId' in vlans[0]

    """
    if devip is not None:
        devid = get_dev_details(devip, auth, url)['id']
    get_dev_vlans_url = "/imcrs/vlan?devId=" + str(devid) + "&start=0&size=5000&total=false"
    f_url = url + get_dev_vlans_url
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            dev_vlans = (json.loads(response.text))
            return dev_vlans['vlan']
        elif response.status_code == 409:
            return {'vlan': 'no vlans'}
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' get_dev_vlans: An Error has occured'


def get_trunk_interfaces(auth, url, devid=None, devip=None):
    """Function takes devId as input to RESTFULL call to HP IMC platform

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param devid: str requires devid of the target device

    :param devip: str of ipv4 address of the target device

    :return: list of dictionaries where each element of the list represents an interface which
    has been configured as a
    VLAN trunk port

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.vlanm import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> trunk_interfaces = get_trunk_interfaces('10', auth.creds, auth.url)

    >>> assert type(trunk_interfaces) is list

    >>> assert len(trunk_interfaces[0]) == 3

    >>> assert 'allowedVlans' in trunk_interfaces[0]

    >>> assert 'ifIndex' in trunk_interfaces[0]

    >>> assert 'pvid' in trunk_interfaces[0]

    >>> get_trunk_interfaces('350', auth.creds, auth.url)
    ['No trunk inteface']
    """
    if devip is not None:
        devid = get_dev_details(devip, auth, url)['id']
    get_trunk_interfaces_url = "/imcrs/vlan/trunk?devId=" + str(devid) + \
                               "&start=1&size=5000&total=false"
    f_url = url + get_trunk_interfaces_url
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            dev_trunk_interfaces = (json.loads(response.text))
            if len(dev_trunk_interfaces) == 2:
                if isinstance(dev_trunk_interfaces['trunkIf'], list):
                    return dev_trunk_interfaces['trunkIf']
                elif isinstance(dev_trunk_interfaces['trunkIf'], dict):
                    return [dev_trunk_interfaces['trunkIf']]
            else:
                dev_trunk_interfaces['trunkIf'] = ["No trunk inteface"]
                return dev_trunk_interfaces['trunkIf']
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' get_trunk_interfaces: An Error has occured'


def get_device_access_interfaces(auth, url, devid=None, devip=None):
    """
    Function takes devid pr devip as input to RESTFUL call to HP IMC platform

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param devid: str requires devid of the target device

    :param devip: str of ipv4 address of the target device

    :return: list of dictionaries where each element of the list represents an interface which
    has been configured as a
    VLAN access port

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.vlanm import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> access_interfaces = get_device_access_interfaces('10', auth.creds, auth.url)

    >>> assert type(access_interfaces) is list

    >>> assert (len(access_interfaces[0])) is 2

    >>> assert 'ifIndex' in access_interfaces[0]

    >>> assert 'pvid' in access_interfaces[0]

    """
    if devip is not None:
        devid = get_dev_details(devip, auth, url)['id']
    get_access_interface_vlan_url = "/imcrs/vlan/access?devId=" + str(devid) + \
                                    "&start=1&size=500&total=false"
    f_url = url + get_access_interface_vlan_url
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            dev_access_interfaces = (json.loads(response.text))
            if len(dev_access_interfaces) == 2:
                return dev_access_interfaces['accessIf']
            else:
                dev_access_interfaces['accessIf'] = ["No access inteface"]
                return dev_access_interfaces['accessIf']
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_device_access_interfaces: An Error has occured"


# Section for Hybrid Interfaces - Applies to Comware Devices only

def get_device_hybrid_interfaces(auth, url, devid=None, devip=None):
    """
    Function takes devId as input to RESTFUL call to HP IMC platform

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param devid: str requires devid of the target device

    :param devip: str of ipv4 address of the target device

    :return: list of dictionaries where each element of the list represents an interface which
    has been configured as a
    VLAN access port

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.vlanm import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> hybrid_interfaces = get_device_hybrid_interfaces('10', auth.creds, auth.url)

    >>> assert type(access_interfaces) is list

    >>> assert (len(access_interfaces[0])) is 2

    >>> assert 'ifIndex' in access_interfaces[0]

    >>> assert 'pvid' in access_interfaces[0]

    """
    if devip is not None:
        devid = get_dev_details(devip, auth, url)['id']
    get_hybrid_interface_vlan_url = "/imcrs/vlan/hybrid?devId=" + str(devid) + \
                                    "&start=1&size=500&total=false"
    f_url = url + get_hybrid_interface_vlan_url
    response = requests.get(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 200:
            dev_hybrid_interfaces = (json.loads(response.text))
            if len(dev_hybrid_interfaces) == 2:
                dev_hybrid = dev_hybrid_interfaces['hybridIf']
                if isinstance(dev_hybrid, dict):
                    dev_hybrid = [dev_hybrid]
                return dev_hybrid
            else:
                dev_hybrid_interfaces['hybridIf'] = ["No hybrid inteface"]
                return dev_hybrid_interfaces['hybridIf']
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_device_hybrid_interfaces: An Error has occured"


def add_hybrid_interface(ifindex, pvid, taggedvlans, untaggedvlans, auth, url, devip=None,
                         devid=None):
    """
    Function takes ifindex, pvid, tagged vlans untagged vlans as input values to add a hybrid
    port to a HPE Comware based switch. These functions only apply to HPE Comware based devices.
    :param ifindex: str ifIndex value of target interface
    :param pvid: str 802.1q value (1-4094) of target VLAN
    :param taggedvlans:  str 802.1q value, seperated by commas, of target tagged VLANs
    :param untaggedvlans:  str 802.1q value, seperated by commas, of target untagged VLANs
    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class
    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass
    :param devid: str requires devid of the target device
    :param devip: str of ipv4 address of the target device
    :return int of http response code
    :rtype int
    """
    if devip is not None:
        devid = get_dev_details(devip, auth, url)['id']
    add_hybrid_interface_url = "/imcrs/vlan/hybrid?devId=" + str(devid) +  \
                               "&start=1&size=500&total=false"
    f_url = url + add_hybrid_interface_url
    payload = '''{"ifIndex": "''' + ifindex + '''",
        "pvid": "''' + pvid + '''",
        "taggedVlans": "''' + taggedvlans + '''",
        "untagVlanFlag": "true",
        "untaggedVlans": "''' + untaggedvlans + '''"
    }'''
    response = requests.post(f_url, auth=auth, data=payload, headers=HEADERS)
    try:
        if response.status_code == 201:
            return 201
        if response.status_code == 409:
            return 409
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_device_hybrid_interfaces: An Error has occured"


def modify_hybrid_interface(ifindex, pvid, taggedvlans, untaggedvlans, auth, url, devip=None,
                            devid=None):
    """
        Function takes ifindex, pvid, tagged vlans untagged vlans as input values to modify a
        hybrid port to a HPE Comware based switch. These functions only apply to HPE Comware
        based devices.
        :param ifindex: str ifIndex value of target interface
        :param pvid: str 802.1q value (1-4094) of target VLAN
        :param taggedvlans:  str 802.1q value, seperated by commas, of target tagged VLANs
        :param untaggedvlans:  str 802.1q value, seperated by commas, of target untagged VLANs
        :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class
        :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass
        :param devid: str requires devid of the target device
        :param devip: str of ipv4 address of the target device
        :return int of http response code
        :rtype int
        """
    if devip is not None:
        devid = get_dev_details(devip, auth, url)['id']
    f_url = url + "/imcrs/vlan/hybrid?devId=" + str(devid) + "&start=1&size=500&total=false"
    payload = '''{"ifIndex": "''' + ifindex + '''",
        "pvid": "''' + pvid + '''",
        "taggedVlans": "''' + taggedvlans + '''",
        "untagVlanFlag": "true",
        "untaggedVlans": "''' + untaggedvlans + '''"
    }'''
    response = requests.put(f_url, auth=auth, data=payload, headers=HEADERS)
    try:
        if response.status_code == 204:
            return 204
        if response.status_code == 409:
            return 409
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_device_hybrid_interfaces: An Error has occured"


def delete_hybrid_interface(ifindex, auth, url, devip=None, devid=None):
    """
     Function takes devip ( ipv4 address ), ifIndex and pvid (vlanid) of specific device and
     802.1q VLAN tag and issues a RESTFUL call to remove the specified VLAN from the target device.

    :param ifindex: str value of ifIndex for a specific interface on the device

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param devid: str requires devid of the target device

    :param devip: str of ipv4 address of the target device

    :return: int of 204 if successful or 409 if not succesful

    :rtype: int

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.vlanm import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> delete_hybrid_interface('9', auth.creds, auth.url, devip='10.101.0.221')
    409

    >>> add_hybrid = add_hybrid_interface('9', '1', '10', '1', auth.creds, auth.url,
                                           devip='10.101.0.221')

    >>> delete_hybrid = delete_hybrid_interface('9', auth.creds, auth.url, devip='10.101.0.221')

    >>> assert type(delete_hybrid) is int

    >>> assert delete_hybrid == 204
    """
    if devip is not None:
        devid = get_dev_details(devip, auth, url)['id']
    f_url = url + "/imcrs/vlan/hybrid?devId=" + devid + "&ifIndex=" + ifindex
    response = requests.delete(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 204:
            return 204
        if response.status_code == 409:
            return 409
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " get_device_hybrid_interfaces: An Error has occured"


# Section for working with Access Interfaces

def set_access_interface_pvid(ifindex, pvid, auth, url, devip=None, devid=None):
    """
    Function takes devip ( ipv4 address ), ifIndex and pvid (vlanid) of specific device and
    802.1q VLAN tag and issues a RESTFUL call to remove the specified VLAN from the target device.

    :param ifindex: str value of ifIndex for a specific interface on the device

    :param pvid:  str value of dot1q VLAN desired to apply to the device

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param devid: str requires devid of the target device

    :param devip: str of ipv4 address of the target device

    :return: int of 204 if successful or 409 if not succesful

    :rtype: int

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.vlanm import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> set_access_int_vlan = set_access_interface_pvid('9', '1', auth.creds, auth.url,
                                                        devip='10.101.0.221')

    >>> set_access_int_vlan = set_access_interface_pvid('9', '10', auth.creds, auth.url,
                                                        devip='10.101.0.221')

    >>> assert type(set_access_int_vlan) is int

    >>> assert set_access_int_vlan == 204

    >>> set_access_int_vlan = set_access_interface_pvid('9', '1', auth.creds, auth.url,
                                                        devip='10.101.0.221')

    """
    if devip is not None:
        devid = get_dev_details(devip, auth, url)['id']
    set_access_interface_pvid_url = "/imcrs/vlan/access?devId=" + devid + "&destVlanId=" + pvid \
                                    + "&ifIndex=" + str(ifindex)
    f_url = url + set_access_interface_pvid_url
    response = requests.put(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 204:
            return 204
        if response.status_code == 409:
            return 409
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " set_access_interface_pvid: An Error has occured"


def get_access_interface_vlan(ifindex, accessinterfacelist):
    """
    Function which takes input of str of ifIndex value for target interface and
    accessinterfacelist ( output of get_device_access_interfaces) to send against IMC REST
    interface. Function returns str representing the PVD of the target interface

    :param ifindex: str object representing the numeric value of the iFindex for the interface

    :param accessinterfacelist: list object, intended to be the output of the
    get_device_access_interfaces function

    :return: str representing the numeric value of the PVID for the target interface.

    :rtype: str

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.vlanm import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> access_interface_list = get_device_access_interfaces(auth.creds, auth.url,
                                                             devip='10.101.0.221')

    >>> get_access_interface_vlan('4', access_interface_list, auth.creds, auth.url)
    '1'
    """
    for i in accessinterfacelist:
        if i['ifIndex'] == ifindex:
            return i['pvid']
        else:
            return "Not an Access Port"


def create_dev_vlan(vlanid, vlan_name, auth, url, devid=None, devip=None):
    """
    function takes devid and vlanid vlan_name of specific device and 802.1q VLAN tag
    and issues a RESTFUL call to add the specified VLAN from the target device. VLAN Name
    MUST be valid on target device.

    :param vlanid:int or str value of target 802.1q VLAN

    :param vlan_name: str value of the target 802.1q VLAN name. MUST be valid name on target device.

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param devid: str requires devid of the target device

    :param devip: str of ipv4 address of the target device

    :return: str HTTP Response code. Should be 201 if successfully created

    :rtype: str

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.vlanm import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> create_dev_vlan = create_dev_vlan('350', '200', 'test vlan', auth.creds, auth.url)


    """
    if devip is not None:
        devid = get_dev_details(devip, auth, url)['id']
    create_dev_vlan_url = "/imcrs/vlan?devId=" + str(devid)
    f_url = url + create_dev_vlan_url
    payload = '''{"vlanId":"%s", "vlanName":"%s"}''' % (str(vlanid), vlan_name)
    response = requests.post(f_url, data=payload, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 201:
            print('Vlan Created')
            return 201
        elif response.status_code == 409:
            print('''Unable to create VLAN.\nVLAN Already Exists\nDevice does not support  VLAN
            function''')
            return 409
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " create_dev_vlan: An Error has occured"


def delete_dev_vlans(vlanid, auth, url, devid=None, devip=None):
    """
    function takes devid and vlanid of specific device and 802.1q VLAN tag and issues a RESTFUL
    call to remove the specified VLAN from the target device.
    :param vlanid:int or str value of target 802.1q VLAN

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: HTTP response object from requests library. Status code should be 204 if Successful

    :param devid: str requires devid of the target device

    :param devip: str of ipv4 address of the target device

    :rtype: requests.models.Response

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.vlanm import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> create_dev_vlan = create_dev_vlan('350', '200', 'test vlan', auth.creds, auth.url)
    """
    if devip is not None:
        devid = get_dev_details(devip, auth, url)['id']
    remove_dev_vlan_url = "/imcrs/vlan/delvlan?devId=" + str(devid) + "&vlanId=" + str(vlanid)
    f_url = url + remove_dev_vlan_url
    response = requests.delete(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 204:
            print('Vlan deleted')
            return response.status_code
        elif response.status_code == 409:
            print('Unable to delete VLAN.\nVLAN does not Exist\nDevice does not support  VLAN '
                  'function')
            return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + " delete_dev_vlans: An Error has occured"



#!/usr/bin/env python3
# coding=utf-8
# author: @netmanchris
# -*- coding: utf-8 -*-
"""
This module contains functions for working with the system operator
capabilities of the HPE IMC NMS platform using the RESTful API

"""

# This section imports required libraries
import json

import requests

from pyhpeimc.auth import HEADERS

# This section deals with HPE IMC Operator Functions



def create_operator(operator, auth, url):
    """
    Function takes input of dictionary operator with the following keys
    operator = { "fullName" : ""   ,
             "sessionTimeout" : "",
             "password" :  "",
             "operatorGroupId" : "",
             "name" : "",
             "desc" : "",
             "defaultAcl" : "",
             "authType"  : ""}
    converts to json and issues a HTTP POST request to the HPE IMC Restful API

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :param operator: dictionary with the required operator key-value pairs as defined above.

    :return:

    :rtype:

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.operator import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> operator = { "fullName" : "test administrator",
                     "sessionTimeout" : "30",
                     "password" :  "password",
                     "operatorGroupId" : "1",
                     "name" : "testadmin",
                     "desc" : "test admin account",
                     "defaultAcl" : "",
                     "authType"  : "0"}

    >>> delete_if_exists = delete_plat_operator('testadmin', auth.creds, auth.url)

    >>> new_operator = create_operator(operator, auth.creds, auth.url)

    >>> assert type(new_operator) is int

    >>> assert new_operator == 201

    >>> fail_operator_create = create_operator(operator, auth.creds, auth.url)

    >>> assert type(fail_operator_create) is int

    >>> assert fail_operator_create == 409

    """
    f_url = url + '/imcrs/plat/operator'
    payload = json.dumps(operator, indent=4)
    response = requests.post(f_url, data=payload, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 409:
            return response.status_code
        elif response.status_code == 201:
            return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' create_operator: An Error has occured'


def set_operator_password(operator, password, auth, url):
    """
    Function to set the password of an existing operator

    :param operator: str Name of the operator account

    :param password: str New password

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: int of 204 if successfull,

    :rtype: int

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.operator import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> operator = { "fullName" : "test administrator", "sessionTimeout" : "30",
                     "password" :  "password","operatorGroupId" : "1",
                     "name" : "testadmin","desc" : "test admin account",
                     "defaultAcl" : "","authType"  : "0"}

    >>> new_operator = create_operator(operator, auth.creds, auth.url)

    >>> set_new_password = set_operator_password('testadmin', 'newpassword', auth.creds, auth.url)

    >>> assert type(set_new_password) is int

    >>> assert set_new_password == 204

       """
    if operator is None:
        operator = input(
            '''\n What is the username you wish to change the password?''')
    oper_id = ''
    authtype = None
    plat_oper_list = get_plat_operator(auth, url)
    for i in plat_oper_list:
        if i['name'] == operator:
            oper_id = i['id']
            authtype = i['authType']
    if oper_id == '':
        return "User does not exist"
    change_pw_url = "/imcrs/plat/operator/"
    f_url = url + change_pw_url + oper_id
    if password is None:
        password = input(
            '''\n ============ Please input the operators new password:\n ============  ''')
    payload = json.dumps({'password': password, 'authType': authtype})
    response = requests.put(f_url, data=payload, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 204:
            # print("Operator:" + operator +
            # " password was successfully changed")
            return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' set_operator_password: An Error has occured'


def get_plat_operator(auth, url):
    """
    Funtion takes no inputs and returns a list of dictionaties of all of the operators currently
    configured on the HPE IMC system

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: list of dictionaries where each element represents one operator

    :rtype: list

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.operator import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> plat_operators = get_plat_operator(auth.creds, auth.url)

    >>> assert type(plat_operators) is list

    >>> assert 'name' in plat_operators[0]

    """
    f_url = url + '/imcrs/plat/operator?start=0&size=1000&orderBy=id&desc=false&total=false'
    try:
        response = requests.get(f_url, auth=auth, headers=HEADERS)
        plat_oper_list = json.loads(response.text)['operator']
        if isinstance(plat_oper_list, dict):
            oper_list = [plat_oper_list]
            return oper_list
        return plat_oper_list
    except requests.exceptions.RequestException as error:
        print("Error:\n" + str(error) + ' get_plat_operator: An Error has occured')
        return "Error:\n" + str(error) + ' get_plat_operator: An Error has occured'


def delete_plat_operator(operator, auth, url):
    """
    Function to set the password of an existing operator
    :param operator: str Name of the operator account

    :param auth: requests auth object #usually auth.creds from auth pyhpeimc.auth.class

    :param url: base url of IMC RS interface #usually auth.url from pyhpeimc.auth.authclass

    :return: int of 204 if successfull

    :rtype: int

    >>> from pyhpeimc.auth import *

    >>> from pyhpeimc.plat.operator import *

    >>> auth = IMCAuth("http://", "10.101.0.203", "8080", "admin", "admin")

    >>> success_delete_operator = delete_plat_operator('testadmin', auth.creds, auth.url)

    >>> assert type(success_delete_operator) is int

    >>> assert success_delete_operator == 204

    >>> fail_delete_operator = delete_plat_operator('testadmin', auth.creds, auth.url)

    >>> assert type(fail_delete_operator) is int

    >>> assert fail_delete_operator == 409

    """
    oper_id = None
    plat_oper_list = get_plat_operator(auth, url)
    for i in plat_oper_list:
        if operator == i['name']:
            oper_id = i['id']
        else:
            oper_id = None
    if oper_id is None:
        # print ("User does not exist")
        return 409
    f_url = url + "/imcrs/plat/operator/" + str(oper_id)
    response = requests.delete(f_url, auth=auth, headers=HEADERS)
    try:
        if response.status_code == 204:
            # print("Operator: " + operator +
            #  " was successfully deleted")
            return response.status_code
    except requests.exceptions.RequestException as error:
        return "Error:\n" + str(error) + ' delete_plat_operator: An Error has occured'



#!/usr/bin/env python3
# coding=utf-8
# author: @netmanchris
# -*- coding: utf-8 -*-
"""
This module contains contains a single function to return the current
version of the pyhpeimc library. The function should return the
current value the library and should be updated with every
new version

"""


def version():
    """Function takes no aruguments and returns a STR value of the current
    version of the library. This value should match
    the value in the test_machine.py
    :return str value of the current version of the library
    :rtype str

    >>> version()
    1.0.44
    """
    print('1.0.44')
